@@ -1,4 +1,4 @@
-# copyright 2004-2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2004-2014 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This file is part of yams.
@@ -19,17 +19,49 @@
 """
 from logilab.common.testlib import TestCase, unittest_main
 
-from yams.reader import SchemaLoader
+from yams.reader import build_schema_from_namespace
+from yams.buildobjs import EntityType, String, SubjectRelation, RelationDefinition
 
+def build_schema():
+
+    class Person(EntityType):
+        firstname = String()
+        knows = SubjectRelation('Person')
+        works_for = SubjectRelation('Company')
+
+    class Student(Person):
+        __specializes_schema__ = True
+
+    class Company(EntityType):
+        name = String()
+
+    class SubCompany(Company):
+        __specializes_schema__ = True
+
+    class Division(Company):
+        __specializes_schema__ = True
+        division_of = SubjectRelation('Company')
+
+    class SubDivision(Division):
+        __specializes_schema__ = True
+
+    # This class doesn't extend the schema
+    class SubSubDivision(SubDivision):
+        pass
+
+    class custom_attr(RelationDefinition):
+        subject = 'Person'
+        object = 'String'
+        __permissions__ = {'read': ('managers', ),
+                           'add': ('managers', ),
+                           'update': ('managers', )}
+
+    return build_schema_from_namespace(locals().items())
 
 
 class SpecializationTC(TestCase):
     def setUp(self):
-        SchemaLoader.main_schema_directory = 'spschema'
-        self.schema = SchemaLoader().load([self.datadir], 'Test')
-
-    def tearDown(self):
-        SchemaLoader.main_schema_directory = 'schema'
+        self.schema = build_schema()
 
     def test_schema_specialization(self):
         schema = self.schema
@@ -153,6 +185,19 @@ class SpecializationTC(TestCase):
                               expected[subjobj])
         self.assertEqual(len(set(expected) - done), 0, 'missing %s' % (set(expected) - done))
 
+    def test_no_more_infered_relations(self):
+        rdef = self.schema['division_of'].rdefs['SubSubDivision', 'SubCompany']
+        self.assertEqual('**', rdef.cardinality)
+        rdef = RelationDefinition('SubSubDivision', 'division_of', 'SubCompany',
+                                  cardinality='1*')
+        # ensure add_relation_def doesn't raise an error
+        self.schema.add_relation_def(rdef)
+        rdef = self.schema['division_of'].rdefs['SubSubDivision', 'SubCompany']
+        self.assertEqual('1*', rdef.cardinality)
+        rdef = self.schema['SubSubDivision'].rdef('division_of', 'subject', 'SubCompany')
+        self.assertEqual('1*', rdef.cardinality)
+        rdef = self.schema['SubCompany'].rdef('division_of', 'object', 'SubSubDivision')
+        self.assertEqual('1*', rdef.cardinality)
 
 if __name__ == '__main__':
     unittest_main()
